@@ -1,5 +1,5 @@
 import traceback
-from typing import AsyncIterable, Iterable
+from typing import AsyncIterable, Iterable, Optional
 
 import aiohttp
 import requests
@@ -7,11 +7,17 @@ from betterproto import Casing
 
 from maitai._config import config
 from maitai._maitai_client import MaitaiClient
-from maitai._types import AsyncChunkQueue, ChunkQueue, EvaluateCallback, QueueIterable
-from maitai_gen.chat import ChatCompletionParams, ChatCompletionRequest
+from maitai._utils import chat_completion_chunk_to_response
+from maitai_common.utils.types import AsyncChunkQueue, ChunkQueue, EvaluateCallback, QueueIterable
+from maitai_gen.chat import ChatCompletionChunk, ChatCompletionParams, ChatCompletionRequest, ChatCompletionResponse, ChatStorageRequest
 from maitai_gen.inference import InferenceStreamResponse
 
 
+class InferenceException(Exception):
+    def __init__(self, msg: str):
+        super().__init__(msg)
+
+
 class Inference(MaitaiClient):
 
     def __init__(self):
@@ -38,16 +44,19 @@ class Inference(MaitaiClient):
         return QueueIterable(q, timeout=timeout)
 
     @classmethod
-    def create_inference_request(cls, application_ref_name, session_id, reference_id, action_type, apply_corrections, evaluation_enabled, completion_params: ChatCompletionParams):
-        infer_request: ChatCompletionRequest = ChatCompletionRequest()
-        infer_request.application_ref_name = application_ref_name
-        infer_request.reference_id = reference_id
-        infer_request.session_id = session_id
-        infer_request.action_type = action_type
-        infer_request.apply_corrections = apply_corrections
-        infer_request.params = completion_params
-        infer_request.evaluation_enabled = evaluation_enabled
-        return infer_request
+    def create_chat_storage_request(cls, session_id, reference_id, action_type, application_ref_name, completion_params: ChatCompletionParams, chat_completion_response: Optional[ChatCompletionResponse],
+                                    final_chunk: Optional[ChatCompletionChunk], content: str):
+        inference_request = cls.create_inference_request(application_ref_name, session_id, reference_id, action_type, False, False, completion_params)
+        if final_chunk:
+            chat_completion_response = chat_completion_chunk_to_response(final_chunk, content)
+
+        return ChatStorageRequest(chat_completion_request=inference_request, chat_completion_response=chat_completion_response)
+
+    @classmethod
+    def store_chat_response(cls, session_id, reference_id, action_type, application_ref_name, completion_params: ChatCompletionParams, chat_completion_response: Optional[ChatCompletionResponse],
+                            final_chunk: Optional[ChatCompletionChunk], content: str):
+        chat_storage_request = cls.create_chat_storage_request(session_id, reference_id, action_type, application_ref_name, completion_params, chat_completion_response, final_chunk, content)
+        cls.run_async(cls.send_storage_request_async(chat_storage_request))
 
     @classmethod
     def send_inference_request(cls, chat_request: ChatCompletionRequest) -> Iterable[InferenceStreamResponse]:
@@ -62,8 +71,6 @@ class Inference(MaitaiClient):
             if response.status_code != 200:
                 print(f"Failed to send inference request. Status code: {response.status_code}. Error: {response.text}")
                 return
-            else:
-                print(f"Successfully sent inference request. Status code: {response.status_code}")
             try:
                 for line in response.iter_lines():
                     if line:
@@ -89,9 +96,9 @@ class Inference(MaitaiClient):
                     error_text = await response.text()
                     print(f"Failed to send evaluation request. Status code: {response.status}. Error: {error_text}")
                     if chunk_queue:
-                        chunk_queue.put("done")
+                        chunk_queue.put(StopIteration())
                     if async_chunk_queue:
-                        await async_chunk_queue.put("done")
+                        await async_chunk_queue.put(StopIteration())
                     return
                 async for line in response.content:
                     if line:
@@ -106,6 +113,20 @@ class Inference(MaitaiClient):
                             except:
                                 traceback.print_exc()
                 if chunk_queue:
-                    chunk_queue.put("done")
+                    chunk_queue.put(StopIteration())
                 if async_chunk_queue:
-                    await async_chunk_queue.put("done")
+                    await async_chunk_queue.put(StopIteration())
+
+    @classmethod
+    async def send_storage_request_async(cls, chat_storage_request: ChatStorageRequest):
+        host = config.maitai_host
+        url = f'{host}/chat/completions/response'
+        headers = {
+            'Content-Type': 'application/json',
+            'x-api-key': config.api_key
+        }
+        try:
+            async with aiohttp.ClientSession(connector=aiohttp.TCPConnector(ssl=False)) as session:
+                return await session.put(url, headers=headers, data=chat_storage_request.to_json(casing=Casing.SNAKE))
+        except:
+            traceback.print_exc()
