@@ -8,11 +8,12 @@ import openai.types.chat as openai_chat_types
 import maitai
 from maitai._config import config
 from maitai._evaluator import Evaluator
+from maitai._inference import InferenceException
 from maitai._openai_types import Body, Headers, Query
-from maitai._types import EvaluateCallback
 from maitai._utils import convert_open_ai_chat_completion_chunk, convert_openai_chat_completion, get_chat_completion_params, required_args
 from maitai_common.utils.proto_utils import openai_messages_to_proto
-from maitai_gen.chat import ChatCompletionChunk, ChatCompletionResponse, ChatMessage, EvaluationContentType
+from maitai_common.utils.types import EvaluateCallback
+from maitai_gen.chat import ChatCompletionChunk, ChatCompletionParams, ChatCompletionResponse, ChatMessage, EvaluationContentType
 from maitai_gen.config import InferenceLocations
 from maitai_gen.inference import InferenceStreamResponse
 
@@ -38,6 +39,8 @@ class MaitaiAsyncOpenAIClient:
     ):
         if maitai_api_key:
             config.initialize(maitai_api_key)
+        if api_key:
+            config.auth_info.override_api_key = api_key
         self.client = openai.AsyncClient(
             api_key=api_key,
             organization=organization,
@@ -138,28 +141,28 @@ class AsyncCompletions:
         if n == openai.NOT_GIVEN:
             n = maitai_config.n
 
+        completion_params = get_chat_completion_params(
+            messages=messages,
+            model=model,
+            frequency_penalty=frequency_penalty,
+            logit_bias=logit_bias,
+            logprobs=logprobs,
+            max_tokens=max_tokens,
+            n=n,
+            presence_penalty=presence_penalty,
+            response_format=response_format,
+            seed=seed,
+            stop=stop,
+            stream=stream,
+            stream_options=stream_options,
+            temperature=temperature,
+            tool_choice=tool_choice,
+            tools=tools,
+            top_logprobs=top_logprobs,
+            top_p=top_p,
+            user=user,
+        )
         if server_side_inference:
-            completion_params = get_chat_completion_params(
-                messages=messages,
-                model=model,
-                frequency_penalty=frequency_penalty,
-                logit_bias=logit_bias,
-                logprobs=logprobs,
-                max_tokens=max_tokens,
-                n=n,
-                presence_penalty=presence_penalty,
-                response_format=response_format,
-                seed=seed,
-                stop=stop,
-                stream=stream,
-                stream_options=stream_options,
-                temperature=temperature,
-                tool_choice=tool_choice,
-                tools=tools,
-                top_logprobs=top_logprobs,
-                top_p=top_p,
-                user=user,
-            )
             response_timeout = None
             if isinstance(timeout, float) or isinstance(timeout, int):
                 response_timeout = timeout
@@ -169,9 +172,13 @@ class AsyncCompletions:
             # ChatCompletion only
             chat_completion: Optional[ChatCompletionResponse] = None
             async for resp in response:
+                if resp.error:
+                    raise InferenceException(resp.error)
                 return resp.chat_completion_response
             return chat_completion
         else:
+            if self.client is None:
+                self.client = openai.OpenAI()
             response = await self.client.chat.completions.create(
                 messages=messages,
                 model=model,
@@ -201,7 +208,7 @@ class AsyncCompletions:
 
             )
             if stream:
-                return _process_async_openai_stream(session_id, reference_id, action_type, application_ref_name, messages, response, evaluation_enabled, callback)
+                return _process_async_openai_stream(session_id, reference_id, action_type, application_ref_name, messages, response, evaluation_enabled, completion_params, callback)
             else:
                 maitai_completion = convert_openai_chat_completion(response)
                 proto_messages = openai_messages_to_proto(messages)
@@ -216,6 +223,19 @@ class AsyncCompletions:
                         content=proto_messages,
                         application_ref_name=application_ref_name,
                         callback=callback,
+                        chat_completion_response=maitai_completion,
+                        completion_params=completion_params,
+                    )
+                else:
+                    maitai.Inference.store_chat_response(
+                        session_id=session_id,
+                        reference_id=reference_id,
+                        action_type=action_type,
+                        application_ref_name=application_ref_name,
+                        chat_completion_response=maitai_completion,
+                        completion_params=completion_params,
+                        final_chunk=None,
+                        content=""
                     )
                 return maitai_completion
 
@@ -227,6 +247,7 @@ async def _process_async_openai_stream(session_id: Union[str, int],
                                        messages: Iterable[openai_chat_types.ChatCompletionMessageParam],
                                        stream: openai.AsyncStream[openai_chat_types.ChatCompletionChunk],
                                        evaluation_enabled: bool,
+                                       chat_completion_params: ChatCompletionParams,
                                        callback: Optional[EvaluateCallback] = None) -> AsyncIterable[ChatCompletionChunk]:
     full_body = ""
     proto_messages = openai_messages_to_proto(messages)
@@ -236,22 +257,38 @@ async def _process_async_openai_stream(session_id: Union[str, int],
         if content is not None:
             full_body += content
         yield maitai_chunk
-        if maitai_chunk.choices[0].finish_reason and evaluation_enabled:
-            proto_messages.append(ChatMessage(role="assistant", content=full_body))
-            await Evaluator.evaluate_async(
-                session_id=session_id,
-                reference_id=reference_id,
-                action_type=action_type,
-                content_type=EvaluationContentType.MESSAGE,
-                content=proto_messages,
-                application_ref_name=application_ref_name,
-                callback=callback
-            )
+        if maitai_chunk.choices[0].finish_reason:
+            if evaluation_enabled:
+                proto_messages.append(ChatMessage(role="assistant", content=full_body))
+                await Evaluator.evaluate_async(
+                    session_id=session_id,
+                    reference_id=reference_id,
+                    action_type=action_type,
+                    content_type=EvaluationContentType.MESSAGE,
+                    content=proto_messages,
+                    application_ref_name=application_ref_name,
+                    callback=callback,
+                    chat_completion_chunk=maitai_chunk,
+                    completion_params=chat_completion_params,
+                )
+            else:
+                maitai.Inference.store_chat_response(
+                    session_id=session_id,
+                    reference_id=reference_id,
+                    action_type=action_type,
+                    application_ref_name=application_ref_name,
+                    completion_params=chat_completion_params,
+                    final_chunk=maitai_chunk,
+                    content=full_body,
+                    chat_completion_response=None,
+                )
             return
 
 
 async def _process_inference_stream_async(stream: AsyncIterable[InferenceStreamResponse]) -> AsyncIterable[ChatCompletionChunk]:
     async for infer_resp in stream:
+        if infer_resp.error:
+            raise InferenceException(infer_resp.error)
         chunk = infer_resp.chat_completion_chunk
         if chunk is not None:
             yield chunk
