@@ -1,8 +1,12 @@
 from maitai_gen.config import Config, InferenceLocations
 
 # TODO consider finding a source of truth where we don't need to deploy everytime a new model comes out
+SERVER_PROVIDERS = ["groq", "openai"]
+CLIENT_PROVIDERS = ["openai"]
+
 MODELS = {
     "llama3-70b-8192": "groq",
+    "llama3-8b-8192": "groq",
     "gpt-4o": "openai",
     "gpt-4o-2024-05-13": "openai",
     "gpt-4-turbo": "openai",
@@ -47,5 +51,13 @@ def get_model_provider(model_name: str):
     return MODELS[model_name]
 
 
+def get_models(providers):
+    models = []
+    for model_name, provider in MODELS.items():
+        if provider in providers:
+            models.append(model_name)
+    return models
+
+
 def get_available_models():
-    return {"models": list(MODELS.keys())}
+    return {"all_models": list(MODELS.keys()), "client_models": get_models(CLIENT_PROVIDERS), "server_models": get_models(SERVER_PROVIDERS)}
